/******************************************************************************/
/*          ______  ___   ______  ___   __________   ______                   */
/*          ___   |/  /   ___   |/  /   ___  ____/   __<  /                   */
/*          __  /|_/ /    __  /|_/ /    __  /_       __  /                    */
/*          _  /  / /     _  /  / /     _  __/       _  /                     */
/*          /_/  /_/      /_/  /_/      /_/          /_/                      */
/*                                                                (2009/2010) */
/******************************************************************************/
/*  Program rjesava dvodimenzionalne odredejne integrale pomocu Trapeznog     */
/*  pravila te sprema rezultate u izlazne datoteke.                           */
/*  Potrebno je mijenjati:                                                    */
/*    - donju granicu integracije  po x (x1)                                  */
/*    - gornju granicu integracije po x (x2)                                  */
/*    - donju granicu integracije  po y (y1)                                  */
/*    - gornju granicu integracije po y (y2)                                  */
/*    - maks. broj podijela segmenta preko kojeg integriramo po x(max_ndx)    */
/*    - broj segmenata preko kojih integriramo po y (ndy)                     */
/*    - vrijednost dobivenu analiticki ako racunamo relativnu pogresku (exact)*/
/*    - podintegralnu funkciju f(x,y)                                         */
/*                                                                            */
/******************************************************************************/

#include<stdio.h>      // Deinirane standardne ulazno, izlazne funkcije
#include<stdlib.h>     // Deinirana nama potrebna funkcija za pauziranje
#include <math.h>      // Definirane potrebne matematicke funkcije
 
#define exact    0.75  // Vrijednost integrala dobivena analiticki

int main(void){
  int i_ndx, max_ndx;			 
  double i_t,rg_t, x1, x2; // Numericke vrijednosti integrala i relativne greske
  FILE *fpt;               // Datoteke za pohranu numerickih vrijednosti
  double trapez (int, double, double); // Trapezno pravilo

  // Otvaranje potrebnih datoteka
  if ((fpt = fopen("trapez.txt","w")) == NULL) {
    printf("Nije moguce pronaci izlaznu datoteku\n");
    system("PAUSE");
    return 0;
  }

  fprintf(fpt,"%52s\n","#          N              Trapez    Relativna greska");
  fprintf(fpt,"%52s\n","#  ---------   -----------------   -----------------");

  // Parametri integracije po x-u
  max_ndx=1001;  // Maksimalni broj podijela po x (neparan za Simpsona)
  x1=0.0;        // Donja granica integracije  po x-u
  x2=1.0;        // Gornja granica integracije po x-u

  // Pozivanje funkcija za numericku integraciju
  for (i_ndx=3; i_ndx<=max_ndx; i_ndx+=4){ //Pocetni mora biti neparan za Simpsona
    i_t = trapez(i_ndx, x1, x2);
    rg_t = fabs((i_t-exact)/exact);
    fprintf(fpt, "%12i%20.9f%20.9f\n", i_ndx, i_t, rg_t);
  }
  printf("\n*************************************************************");
  printf("\n* Numericka rjesenja pohranjena su u izlaznim datotekama!\n");
  printf("\n*************************************************************\n");
  fclose(fpt);
  system("PAUSE");
  return 0;
}

/* Trapezno pravilo za numericku integraciju */
double trapez (int ndx, double x1, double x2){
  int idx;
  double dx, sum=0., x;
  double finteg(double);	   // Podintegralna funkcija koja sadrzi integral
  dx = ((x2-x1) / (ndx-1));
  // Doprinos unutarnjih segmenata
  for (idx=2; idx<ndx; idx++) {
    x    = x1 + dx * (idx-1);
    sum += finteg(x)*dx;
  }
  // Doprinos krajnjih segmenata
  sum += 0.5 *(finteg(x1) + finteg(x2)) * dx;
  return (sum);
}      

/* Definicija podintegralne funkcije koja sadrzava integral*/
double finteg (double x){
   int ndy; 
   double result, y1, y2;    
   double podtrapez  (int, double, double, double);

   //Parametri integracije po y-u
   ndy=101;
   y1=0.;
   y2=1.-x;

   //Pozivanje funkcija za numericku integraciju
   result=podtrapez(ndy,y1,y2,x);
   
   return (result);    
}

/* Trapezno pravilo za numericku integraciju */
double podtrapez (int ndy, double y1, double y2, double x){
   int idy;
   double dy, sum=0., y;
   double f (double, double);

   dy = ((y2-y1)/(ndy-1));
   for (idy=2; idy<ndy; idy++){
      y    = y1 + dy*(idy-1);     
      sum += f(x,y)*dy;
   }
   sum += 0.5 *(f(x,y1) + f(x,y2)) * dy;

   return (sum);
}      

/* Definicija podintegralne funkcije */
double f (double x, double y){        
   return (2*x*x+y*y+1.);               
}
